MERMAID_CDN = "https://cdnjs.cloudflare.com/ajax/libs/mermaid/%s/mermaid.min.js"

DEFAULT_VERSION = "9.4.3"  # default to latest stable version
DEFAULT_THEME = "default"  # use the mermaid 'default' theme
DEFAULT_USE_CDN = False  # use local mermaid.js by default

__all__ = [
    "DEFAULT_USE_CDN",
    "DEFAULT_VERSION",
    "DEFAULT_THEME",
    "MERMAID_CDN",
]
